/****************************************************************************
** mainwin.h - the main window.  holds contactlist and buttons.
** Copyright (C) 2001, 2002  Justin Karneges
**
** This program is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public License
** as published by the Free Software Foundation; either version 2
** of the License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
**
****************************************************************************/

#ifndef MAINWIN_H
#define MAINWIN_H

#include<qdialog.h>
#include<qwidget.h>
#include<qpushbutton.h>
#include<qtoolbutton.h>
#include<qpopupmenu.h>
#include<qlayout.h>
#include<qlabel.h>
#include<qptrvector.h>
#include<qtooltip.h>
#include"kmp/types.h"
#include"contactview.h"
#include"trayicon.h"
#include"iconset.h"


class PsiCon;
class PsiAccount;
class QIconSet;

class AboutDlg : public QDialog
{
	Q_OBJECT
public:
	AboutDlg(const QPixmap &icon, const QString &content, QWidget *par=0);
};

class MPushButton : public AnimatedButton
{
	Q_OBJECT
public:
	MPushButton(QWidget *parent=0, const char *name=0);

	void setText(const QString &);

protected:
	// reimplemented
	void drawButtonLabel(QPainter *);

private:
	bool hasTip;
};

class MToolButton : public QToolButton
{
	Q_OBJECT
public:
	MToolButton(QWidget *parent=0, const char *name=0);

public slots:
	void setPressed(bool);
};

class MLabel : public QLabel
{
	Q_OBJECT
public:
	MLabel(QWidget *parent=0, const char *name=0);

protected:
	// reimplemented
	void mouseReleaseEvent(QMouseEvent *);
	void mouseDoubleClickEvent(QMouseEvent *);

signals:
	void clicked(int);
	void doubleClicked();
};

class MTray : public QObject
{
	Q_OBJECT
public:
	MTray(const QImage &icon, const QString &tip, QPopupMenu *popup, QObject *parent=0);
	~MTray();

	void setToolTip(const QString &);
	void setImage(const QImage &);
	void setAnim(Anim *);
	bool isAnimating() const;

signals:
	void clicked(const QPoint &, int);
	void doubleClicked(const QPoint &);
	void closed();

public slots:
	void show();
	void hide();

private slots:
	void animate();

private:
	TrayIcon *ti;
	QTimer *trayTimer;
	Anim *anim;
	int animStep;

	QPixmap makeIcon(const QImage &_in);
};

class MAction : public QObject
{
	Q_OBJECT
public:
	MAction(const QIconSet &, const QString &, int id, PsiCon *, QObject *parent=0);
	MAction(const QString &, int id, PsiCon *, QObject *parent=0);
	~MAction();

	void addTo(QPopupMenu *);
	void removeFrom(QPopupMenu *);
	int id() const;

signals:
	void activated(PsiAccount *, int);

private slots:
	void itemActivated();
	void menuItemActivated(int);
	void popup_destroyed();
	void numAccountsChanged();

private:
	class Private;
	class MenuItem;
	Private *d;

	void init(const QIconSet &i, const QString &s, int id, PsiCon *psi);
	void insertItem(MenuItem *, int index=-1);
	void doActivate(PsiAccount *);
};

class MainWin : public QWidget
{
	Q_OBJECT
public:
	MainWin(bool onTop, bool asTool, PsiCon *, const char *name=0);
	~MainWin();

	void setWindowOpts(bool onTop, bool asTool);
	void setUseDock(bool);
	void setInfo(const QString &);
	void setUsingSSL(bool);

	QVBoxLayout *vb_main;
	QHBoxLayout *hb_status;

	ContactView *cvlist;
	MLabel *lb_info;
	QLabel *lb_ssl;
	bool usingSSL;
	QPushButton *pb_options;
	MPushButton *pb_status;

	PsiCon *psi;

	Anim *nextAnim;
	int nextAmount;

private:
	bool onTop, asTool;
	QPopupMenu *statusMenu, *optionsMenu;
	int sbState;
	QString nickname;
	MToolButton *tb_offline, *tb_away, *tb_agents;
	MTray *tray;
	QPopupMenu *trayMenu;
	QString statusTip;

	MAction *lw_act, *add_act, *man_act;

	bool askQuit();

	void updateCaption();
	void setToolBarIcons();
	void updateTray();

	QString infoString;

protected:
	void closeEvent(QCloseEvent *);
	void keyPressEvent(QKeyEvent *);

signals:
	void statusChanged(int);
	void changeProfile();
	void blankMessage();
	void closeProgram();
	void doOptions();
	void doManageAccounts();
	void doGroupChat();
	void doFileSharing();
	void accountInfo();
	void recvNextEvent();

private slots:
	void buildStatusMenu();
	void buildOptionsMenu();
	void buildTrayMenu();

	void activatedStatusMenu(int);
	void activatedOptionsMenu(int);

	void trayClicked(const QPoint &, int);
	void trayDoubleClicked();
	void trayShow();
	void trayHide();

	void doRecvNextEvent();
	void statusClicked(int);
	void tryCloseProgram();

	void numAccountsChanged();
	void testme();
	void activatedAccOption(PsiAccount *, int);

public slots:
	void showNoFocus();

	void decorateButton(int);
	void updateReadNext(Anim *nextAnim, int nextAmount);

	void optionsUpdate();

private:
	int lastStatus;

	QString numEventsString(int) const;
};

#endif
