/*
 * psievent.h - events
 * Copyright (C) 2001, 2002  Justin Karneges
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifndef PSIEVENT_H
#define PSIEVENT_H

#include<qdatetime.h>
#include"kmp/jid.h"
#include"kmp/message.h"

using namespace Jabber;

class PsiEvent
{
public:
	enum { Message, Auth };
	PsiEvent();
	virtual ~PsiEvent()=0;

	virtual int type() const=0;
	virtual Jid from() const=0;

	Jid jid() const;
	void setJid(const Jid &);

	bool originLocal() const { return v_originLocal; }
	bool late() const { return v_late; }
	QDateTime timeStamp() const { return v_ts; }

	void setOriginLocal(bool b) { v_originLocal = b; }
	void setLate(bool b) { v_late = b; }
	void setTimeStamp(const QDateTime &t) { v_ts = t; }

private:
	bool v_originLocal, v_late;
	QDateTime v_ts;
	Jid v_jid;
};

// normal, chat, error, headline, etc
class MessageEvent : public PsiEvent
{
public:
	MessageEvent();
	MessageEvent(const Jabber::Message &);
	~MessageEvent();

	int type() const { return Message; }
	Jid from() const { return v_m.from(); }

	bool sentToChatWindow() const { return v_sentToChatWindow; }
	const Jabber::Message & message() const { return v_m; }

	void setSentToChatWindow(bool b) { v_sentToChatWindow = b; }
	void setMessage(const Jabber::Message &m) { v_m = m; setTimeStamp(v_m.timeStamp()); setLate(v_m.spooled()); }

private:
	Jabber::Message v_m;
	bool v_sentToChatWindow;
};

// subscribe, subscribed, unsubscribe, unsubscribed
class AuthEvent : public PsiEvent
{
public:
	AuthEvent(const Jid &j, const QString &authType)
	{
		v_from = j;
		v_at = authType;
	}
	~AuthEvent()
	{
	}

	int type() const { return Auth; }
	Jid from() const { return v_from; }

	QString authType() const { return v_at; }

private:
	Jid v_from;
	QString v_at;
};

#endif
